@@ -27,7 +27,7 @@ enum EventType
         EVENT_GOTOSONG,
         EVENT_GOTOBEGINNINGOFPLAYLIST,
         EVENT_GOTOENDOFPLAYLIST,
-        EVENT_TOGGLEPROFILECOLORS,
+        EVENT_CYCLECOLORMODE,
         EVENT_SCROLLNEXT,
         EVENT_SCROLLPREV,
         EVENT_SEEKBACK,
@@ -45,7 +45,9 @@ enum EventType
         EVENT_MOVESONGDOWN,
         EVENT_ENQUEUEANDPLAY,
         EVENT_STOP,
-        EVENT_SORTLIBRARY
+        EVENT_SORTLIBRARY,
+        EVENT_CYCLETHEMES,
+        EVENT_TOGGLENOTIFICATIONS
 };
 
 struct Event
