@@ -13,7 +13,9 @@ typedef struct
 } PixelData;
 #endif
 
-int printInAscii(const char *pathToImgFile, int height);
+int printInAsciiCentered(const char *pathToImgFile, int height);
+
+int printInAscii(int indentation, const char *pathToImgFile, int height);
 
 float calcAspectRatio(void);
 
@@ -21,8 +23,12 @@ unsigned char *getBitmap(const char *image_path, int *width, int *height);
 
 void printSquareBitmapCentered(unsigned char *pixels, int width, int height, int baseHeight);
 
+void printSquareBitmap(int row, int col, unsigned char *pixels, int width, int height, int baseHeight);
+
 int getCoverColor(unsigned char *pixels, int width, int height, unsigned char *r, unsigned char *g, unsigned char *b);
 
+float getAspectRatio();
+
 #ifdef CHAFA_VERSION_1_16
 gboolean retire_passthrough_workarounds_tmux(void);
 #endif
