@@ -2,8 +2,8 @@
 #define MA_NO_ENGINE
 #define MINIAUDIO_IMPLEMENTATION
 
-#include <miniaudio.h>
 #include "sound.h"
+#include <miniaudio.h>
 
 /*
 
@@ -25,7 +25,9 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
 {
         char *filePath = NULL;
 
-        SongData *songData = (pAudioData->currentFileIndex == 0) ? pUserData->songdataA : pUserData->songdataB;
+        SongData *songData = (pAudioData->currentFileIndex == 0)
+                                 ? pUserData->songdataA
+                                 : pUserData->songdataB;
 
         if (songData == NULL)
         {
@@ -34,6 +36,11 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
 
         filePath = songData->filePath;
 
+        if (filePath == NULL)
+        {
+                return MA_ERROR;
+        }
+
         pAudioData->pUserData = pUserData;
         pAudioData->currentPCMFrame = 0;
         pAudioData->restart = false;
@@ -47,7 +54,8 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                 pAudioData->format = first->outputFormat;
                 pAudioData->channels = first->outputChannels;
                 pAudioData->sampleRate = first->outputSampleRate;
-                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
+                ma_data_source_get_length_in_pcm_frames(
+                    first, &(pAudioData->totalFrames));
         }
         else if (pathEndsWith(filePath, "opus"))
         {
@@ -56,8 +64,11 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 ma_libopus *first = getFirstOpusDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                ma_libopus_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
+                ma_libopus_ds_get_data_format(
+                    first, &(pAudioData->format), &(pAudioData->channels),
+                    &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(
+                    first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -69,8 +80,11 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 ma_libvorbis *first = getFirstVorbisDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                ma_libvorbis_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
+                ma_libvorbis_ds_get_data_format(
+                    first, &(pAudioData->format), &(pAudioData->channels),
+                    &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(
+                    first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -82,8 +96,11 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 ma_webm *first = getFirstWebmDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                ma_webm_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
+                ma_webm_ds_get_data_format(
+                    first, &(pAudioData->format), &(pAudioData->channels),
+                    &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(
+                    first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -97,8 +114,11 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 m4a_decoder *first = getFirstM4aDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                m4a_decoder_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
+                m4a_decoder_ds_get_data_format(
+                    first, &(pAudioData->format), &(pAudioData->channels),
+                    &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(
+                    first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -114,18 +134,19 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
         return MA_SUCCESS;
 }
 
-int createDevice(UserData *userData, ma_device *device, ma_context *context, ma_data_source_vtable *vtable, ma_device_data_proc callback)
+int createDevice(UserData *userData, ma_device *device, ma_context *context,
+                 ma_data_source_vtable *vtable, ma_device_data_proc callback)
 {
         ma_result result;
 
-        ma_data_source_uninit(&audioData);
         result = initFirstDatasource(&audioData, userData);
         if (result != MA_SUCCESS)
                 return -1;
 
         audioData.base.vtable = vtable;
 
-        ma_device_config deviceConfig = ma_device_config_init(ma_device_type_playback);
+        ma_device_config deviceConfig =
+            ma_device_config_init(ma_device_type_playback);
         deviceConfig.playback.format = audioData.format;
         deviceConfig.playback.channels = audioData.channels;
         deviceConfig.sampleRate = audioData.sampleRate;
@@ -134,25 +155,34 @@ int createDevice(UserData *userData, ma_device *device, ma_context *context, ma_
 
         result = ma_device_init(context, &deviceConfig, device);
         if (result != MA_SUCCESS)
+        {
                 return -1;
+        }
 
         setVolume(getCurrentVolume());
 
         result = ma_device_start(device);
         if (result != MA_SUCCESS)
+        {
+                ma_device_uninit(device);
                 return -1;
+        }
 
         appState.uiState.doNotifyMPRISPlaying = true;
 
         return 0;
 }
 
-int builtin_createAudioDevice(UserData *userData, ma_device *device, ma_context *context, ma_data_source_vtable *vtable)
+int builtin_createAudioDevice(UserData *userData, ma_device *device,
+                              ma_context *context,
+                              ma_data_source_vtable *vtable)
 {
-        return createDevice(userData, device, context, vtable, builtin_on_audio_frames);
+        return createDevice(userData, device, context, vtable,
+                            builtin_on_audio_frames);
 }
 
-int vorbis_createAudioDevice(UserData *userData, ma_device *device, ma_context *context)
+int vorbis_createAudioDevice(UserData *userData, ma_device *device,
+                             ma_context *context)
 {
         ma_result result;
 
@@ -163,7 +193,8 @@ int vorbis_createAudioDevice(UserData *userData, ma_device *device, ma_context *
                 return -1;
         }
         ma_libvorbis *vorbis = getFirstVorbisDecoder();
-        ma_device_config deviceConfig = ma_device_config_init(ma_device_type_playback);
+        ma_device_config deviceConfig =
+            ma_device_config_init(ma_device_type_playback);
 
         deviceConfig.playback.format = vorbis->format;
         deviceConfig.playback.channels = audioData.channels;
@@ -193,18 +224,21 @@ int vorbis_createAudioDevice(UserData *userData, ma_device *device, ma_context *
 }
 
 #ifdef USE_FAAD
-int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *context)
+int m4a_createAudioDevice(UserData *userData, ma_device *device,
+                          ma_context *context)
 {
         ma_result result;
 
         result = initFirstDatasource(&audioData, userData);
         if (result != MA_SUCCESS)
         {
-                setErrorMessage("M4a type not supported.");
+                if (!hasErrorMessage())
+                        setErrorMessage("M4a type not supported.");
                 return -1;
         }
         m4a_decoder *decoder = getFirstM4aDecoder();
-        ma_device_config deviceConfig = ma_device_config_init(ma_device_type_playback);
+        ma_device_config deviceConfig =
+            ma_device_config_init(ma_device_type_playback);
 
         deviceConfig.playback.format = decoder->format;
         deviceConfig.playback.channels = audioData.channels;
@@ -235,7 +269,8 @@ int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *con
 }
 #endif
 
-int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *context)
+int opus_createAudioDevice(UserData *userData, ma_device *device,
+                           ma_context *context)
 {
         ma_result result;
 
@@ -247,7 +282,8 @@ int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *co
         }
         ma_libopus *opus = getFirstOpusDecoder();
 
-        ma_device_config deviceConfig = ma_device_config_init(ma_device_type_playback);
+        ma_device_config deviceConfig =
+            ma_device_config_init(ma_device_type_playback);
 
         deviceConfig.playback.format = opus->format;
         deviceConfig.playback.channels = audioData.channels;
@@ -276,7 +312,8 @@ int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *co
         return 0;
 }
 
-int webm_createAudioDevice(UserData *userData, ma_device *device, ma_context *context)
+int webm_createAudioDevice(UserData *userData, ma_device *device,
+                           ma_context *context)
 {
         ma_result result;
 
@@ -287,7 +324,8 @@ int webm_createAudioDevice(UserData *userData, ma_device *device, ma_context *co
                 return -1;
         }
         ma_webm *webm = getFirstWebmDecoder();
-        ma_device_config deviceConfig = ma_device_config_init(ma_device_type_playback);
+        ma_device_config deviceConfig =
+            ma_device_config_init(ma_device_type_playback);
 
         deviceConfig.playback.format = audioData.format;
         deviceConfig.playback.channels = audioData.channels;
@@ -346,7 +384,8 @@ int calcAvgBitRate(double duration, const char *filePath)
         int avgBitRate = 0;
 
         if (duration > 0.0)
-                avgBitRate = (int)((fileSize * 8.0) / duration / 1000.0); // use 1000 for kbps
+                avgBitRate = (int)((fileSize * 8.0) / duration /
+                                   1000.0); // use 1000 for kbps
 
         return avgBitRate;
 }
@@ -360,9 +399,12 @@ int switchAudioImplementation(void)
                 return 0;
         }
 
-        enum AudioImplementation currentImplementation = getCurrentImplementationType();
+        enum AudioImplementation currentImplementation =
+            getCurrentImplementationType();
 
-        userData.currentSongData = (audioData.currentFileIndex == 0) ? userData.songdataA : userData.songdataB;
+        userData.currentSongData = (audioData.currentFileIndex == 0)
+                                       ? userData.songdataA
+                                       : userData.songdataB;
 
         char *filePath = NULL;
 
@@ -378,7 +420,8 @@ int switchAudioImplementation(void)
                 {
                         if (!tryAgain)
                         {
-                                setCurrentFileIndex(&audioData, 1 - audioData.currentFileIndex);
+                                setCurrentFileIndex(
+                                    &audioData, 1 - audioData.currentFileIndex);
                                 tryAgain = true;
                                 switchAudioImplementation();
                                 return 0;
@@ -404,23 +447,27 @@ int switchAudioImplementation(void)
 
                 getFileInfo(filePath, &sampleRate, &channels, &format);
 
-                bool sameFormat = (decoder != NULL && (sampleRate == decoder->outputSampleRate &&
-                                                       channels == decoder->outputChannels &&
-                                                       format == decoder->outputFormat));
+                bool sameFormat = (decoder != NULL &&
+                                   (sampleRate == decoder->outputSampleRate &&
+                                    channels == decoder->outputChannels &&
+                                    format == decoder->outputFormat));
 
                 if (pathEndsWith(filePath, ".mp3") && userData.currentSongData)
                 {
-                        int avgBitRate = calcAvgBitRate(userData.currentSongData->duration, filePath);
+                        int avgBitRate = calcAvgBitRate(
+                            userData.currentSongData->duration, filePath);
 
                         if (avgBitRate > 320)
                                 avgBitRate = 320;
 
-                        userData.currentSongData->avgBitRate = audioData.avgBitRate = avgBitRate;
+                        userData.currentSongData->avgBitRate =
+                            audioData.avgBitRate = avgBitRate;
                 }
                 else
                         audioData.avgBitRate = 0;
 
-                if (isRepeatEnabled() || !(sameFormat && currentImplementation == BUILTIN))
+                if (isRepeatEnabled() ||
+                    !(sameFormat && currentImplementation == BUILTIN))
                 {
                         setImplSwitchReached();
 
@@ -435,7 +482,9 @@ int switchAudioImplementation(void)
 
                         audioData.sampleRate = sampleRate;
 
-                        int result = builtin_createAudioDevice(&userData, getDevice(), &context, &builtin_file_data_source_vtable);
+                        int result = builtin_createAudioDevice(
+                            &userData, getDevice(), &context,
+                            &builtin_file_data_source_vtable);
 
                         if (result < 0)
                         {
@@ -465,16 +514,21 @@ int switchAudioImplementation(void)
                 ma_channel nChannelMap[MA_MAX_CHANNELS];
                 ma_libopus *decoder = getCurrentOpusDecoder();
 
-                getOpusFileInfo(filePath, &format, &channels, &sampleRate, channelMap);
+                getOpusFileInfo(filePath, &format, &channels, &sampleRate,
+                                channelMap);
 
                 if (decoder != NULL)
-                        ma_libopus_ds_get_data_format(decoder, &nFormat, &nChannels, &nSampleRate, nChannelMap, MA_MAX_CHANNELS);
+                        ma_libopus_ds_get_data_format(
+                            decoder, &nFormat, &nChannels, &nSampleRate,
+                            nChannelMap, MA_MAX_CHANNELS);
 
-                bool sameFormat = (decoder != NULL && (format == decoder->format &&
-                                                       channels == nChannels &&
-                                                       sampleRate == nSampleRate));
+                bool sameFormat =
+                    (decoder != NULL &&
+                     (format == decoder->format && channels == nChannels &&
+                      sampleRate == nSampleRate));
 
-                if (isRepeatEnabled() || !(sameFormat && currentImplementation == OPUS))
+                if (isRepeatEnabled() ||
+                    !(sameFormat && currentImplementation == OPUS))
                 {
                         setImplSwitchReached();
 
@@ -490,7 +544,8 @@ int switchAudioImplementation(void)
                         audioData.sampleRate = sampleRate;
                         audioData.avgBitRate = 0;
 
-                        int result = opus_createAudioDevice(&userData, getDevice(), &context);
+                        int result = opus_createAudioDevice(
+                            &userData, getDevice(), &context);
 
                         if (result < 0)
                         {
@@ -520,21 +575,28 @@ int switchAudioImplementation(void)
                 ma_channel nChannelMap[MA_MAX_CHANNELS];
                 ma_libvorbis *decoder = getCurrentVorbisDecoder();
 
-                getVorbisFileInfo(filePath, &format, &channels, &sampleRate, channelMap);
+                getVorbisFileInfo(filePath, &format, &channels, &sampleRate,
+                                  channelMap);
 
                 if (decoder != NULL)
-                        ma_libvorbis_ds_get_data_format(decoder, &nFormat, &nChannels, &nSampleRate, nChannelMap, MA_MAX_CHANNELS);
+                        ma_libvorbis_ds_get_data_format(
+                            decoder, &nFormat, &nChannels, &nSampleRate,
+                            nChannelMap, MA_MAX_CHANNELS);
 
-                bool sameFormat = (decoder != NULL && (format == decoder->format &&
-                                                       channels == nChannels &&
-                                                       sampleRate == nSampleRate));
+                bool sameFormat =
+                    (decoder != NULL &&
+                     (format == decoder->format && channels == nChannels &&
+                      sampleRate == nSampleRate));
 
                 if (userData.currentSongData)
-                        userData.currentSongData->avgBitRate = audioData.avgBitRate = calcAvgBitRate(userData.currentSongData->duration, filePath);
+                        userData.currentSongData->avgBitRate =
+                            audioData.avgBitRate = calcAvgBitRate(
+                                userData.currentSongData->duration, filePath);
                 else
                         audioData.avgBitRate = 0;
 
-                if (isRepeatEnabled() || !(sameFormat && currentImplementation == VORBIS))
+                if (isRepeatEnabled() ||
+                    !(sameFormat && currentImplementation == VORBIS))
                 {
                         setImplSwitchReached();
 
@@ -549,7 +611,8 @@ int switchAudioImplementation(void)
 
                         audioData.sampleRate = sampleRate;
 
-                        int result = vorbis_createAudioDevice(&userData, getDevice(), &context);
+                        int result = vorbis_createAudioDevice(
+                            &userData, getDevice(), &context);
 
                         if (result < 0)
                         {
@@ -579,21 +642,27 @@ int switchAudioImplementation(void)
                 ma_channel nChannelMap[MA_MAX_CHANNELS];
                 ma_webm *decoder = getCurrentWebmDecoder();
 
-                getWebmFileInfo(filePath, &format, &channels, &sampleRate, channelMap);
+                getWebmFileInfo(filePath, &format, &channels, &sampleRate,
+                                channelMap);
 
                 if (decoder != NULL)
-                        ma_webm_ds_get_data_format(decoder, &nFormat, &nChannels, &nSampleRate, nChannelMap, MA_MAX_CHANNELS);
+                        ma_webm_ds_get_data_format(
+                            decoder, &nFormat, &nChannels, &nSampleRate,
+                            nChannelMap, MA_MAX_CHANNELS);
 
                 bool sameFormat = false;
 
                 // FIXME: Gapless/chaining of decoders disabled for now
-                //bool sameFormat = (decoder != NULL && (format == decoder->format &&
-                //                                       channels == nChannels &&
-                //                                       sampleRate == nSampleRate));
+                // bool sameFormat = (decoder != NULL && (format ==
+                // decoder->format &&
+                //                                       channels == nChannels
+                //                                       && sampleRate ==
+                //                                       nSampleRate));
 
                 audioData.avgBitRate = 0;
 
-                if (isRepeatEnabled() || !(sameFormat && currentImplementation == WEBM))
+                if (isRepeatEnabled() ||
+                    !(sameFormat && currentImplementation == WEBM))
                 {
                         setImplSwitchReached();
 
@@ -608,7 +677,8 @@ int switchAudioImplementation(void)
 
                         audioData.sampleRate = sampleRate;
 
-                        int result = webm_createAudioDevice(&userData, getDevice(), &context);
+                        int result = webm_createAudioDevice(
+                            &userData, getDevice(), &context);
 
                         if (result < 0)
                         {
@@ -641,21 +711,27 @@ int switchAudioImplementation(void)
                 m4a_decoder *decoder = getCurrentM4aDecoder();
                 k_m4adec_filetype fileType = k_unknown;
 
-                getM4aFileInfo(filePath, &format, &channels, &sampleRate, channelMap, &avgBitRate, &fileType);
+                getM4aFileInfo(filePath, &format, &channels, &sampleRate,
+                               channelMap, &avgBitRate, &fileType);
 
                 if (decoder != NULL)
-                        m4a_decoder_ds_get_data_format(decoder, &nFormat, &nChannels, &nSampleRate, nChannelMap, MA_MAX_CHANNELS);
+                        m4a_decoder_ds_get_data_format(
+                            decoder, &nFormat, &nChannels, &nSampleRate,
+                            nChannelMap, MA_MAX_CHANNELS);
 
-                bool sameFormat = (decoder != NULL && (format == decoder->format &&
-                                                       channels == nChannels &&
-                                                       sampleRate == nSampleRate &&
-                                                       decoder->fileType == fileType &&
-                                                       decoder->fileType != k_rawAAC));
+                bool sameFormat =
+                    (decoder != NULL &&
+                     (format == decoder->format && channels == nChannels &&
+                      sampleRate == nSampleRate &&
+                      decoder->fileType == fileType &&
+                      decoder->fileType != k_rawAAC));
 
                 if (userData.currentSongData)
-                        userData.currentSongData->avgBitRate = audioData.avgBitRate = avgBitRate;
+                        userData.currentSongData->avgBitRate =
+                            audioData.avgBitRate = avgBitRate;
 
-                if (isRepeatEnabled() || !(sameFormat && currentImplementation == M4A))
+                if (isRepeatEnabled() ||
+                    !(sameFormat && currentImplementation == M4A))
                 {
                         setImplSwitchReached();
 
@@ -670,7 +746,8 @@ int switchAudioImplementation(void)
 
                         audioData.sampleRate = sampleRate;
 
-                        int result = m4a_createAudioDevice(&userData, getDevice(), &context);
+                        int result = m4a_createAudioDevice(
+                            &userData, getDevice(), &context);
 
                         if (result < 0)
                         {
